package OpenTracing::Implementation::Test::Scope;

our $VERSION = 'v0.102.1';

use Moo;

with 'OpenTracing::Role::Scope';

1;



=head1 NAME

OpenTracing::Implementation::Test::Scope - OpenTracing Test for Scope



=head1 AUTHOR

Szymon Nieznanski <snieznanski@perceptyx.com>



=head1 COPYRIGHT AND LICENSE

'Test::OpenTracing::Integration'
is Copyright (C) 2019 .. 2020, Perceptyx Inc

This library is free software; you can redistribute it and/or modify it under
the terms of the Artistic License 2.0.

This package is distributed in the hope that it will be useful, but it is
provided "as is" and without any express or implied warranties.

For details, see the full text of the license in the file LICENSE.


=cut
